#!/bin/python
# -*- coding: utf-8 -*-
"""
dump.py
Dump data to file in selected format.

Author: GrimAndGreedy
License: MIT
"""

import os

def make_list_unique(l:list) -> list:
    """ Ensure each of the strings in a list is unique by numbering identical strings. """
    result = []
    for i in l:
        if i not in result:
            result.append(i)
        else:
            result[-1] += f'_({len(result)-1})'
            result.append(i)
    return result

def dump_state(function_data:dict, file_path:str) -> None:
    """ Dump state of Picker to file. """

    import dill as pickle
    exclude_keys =  ["refresh_function", "get_data_startup", "get_new_data", "auto_refresh"]
    function_data = {key: val for key, val in function_data.items() if key not in exclude_keys}
    with open(os.path.expandvars(os.path.expanduser(file_path)), 'wb') as f:
        pickle.dump(function_data, f)

def dump_data(function_data:dict, file_path:str, format="pickle") -> str:
    """ Dump data from a Picker object. Returns whether there was an error. """
    include_keys = ["items", "header"]
    function_data = {key: val for key, val in function_data.items() if key in include_keys }

    try:
        if format == "pickle":
            import dill as pickle
            with open(os.path.expandvars(os.path.expanduser(file_path)), 'wb') as f:
                pickle.dump(function_data, f)
        elif format == "csv":
            import csv
            with open(os.path.expandvars(os.path.expanduser(file_path)), mode='w', newline='') as f:
                writer = csv.writer(f)
                for row in [function_data["header"]] + function_data["items"]:
                    writer.writerow(row)
        elif format == "tsv":
            import csv
            with open(os.path.expandvars(os.path.expanduser(file_path)), mode='w', newline='') as f:
                writer = csv.writer(f, delimiter='\t')
                for row in [function_data["header"]] + function_data["items"]:
                    writer.writerow(row)

        elif format == "json":
            import json
            with open(os.path.expandvars(os.path.expanduser(file_path)), mode='w') as f:
                json.dump([function_data["header"]]+ function_data["items"], f, indent=4)

        elif format == "feather":
            import pyarrow as pa
            import pandas as pd
            import pyarrow.feather as feather
            table = pa.Table.from_pandas(pd.DataFrame(function_data["items"], columns=make_list_unique(function_data["header"])))
            feather.write_feather(table, os.path.expandvars(os.path.expanduser(file_path)))
            
        elif format == "parquet":
            import pyarrow as pa
            import pyarrow.parquet as pq
            import pandas as pd
            table = pa.Table.from_pandas(pd.DataFrame(function_data["items"], columns=make_list_unique(function_data["header"])))

            pq.write_table(table, os.path.expandvars(os.path.expanduser(file_path)))
        elif format == "msgpack":
            import msgpack as mp
            with open(os.path.expandvars(os.path.expanduser(file_path)), mode='wb') as f:
                mp.dump([function_data["header"]]+ function_data["items"], f)
    except Exception as e:
        return str(e)
    return ""

            

def load_state(file_path:str) -> dict:
    """ Load Picker state from dump. """
    import dill as pickle
    with open(os.path.expandvars(os.path.expanduser(file_path)), 'rb') as f:
        loaded_data = pickle.load(f)
    return loaded_data




#!/bin/python
# -*- coding: utf-8 -*-
"""
options_selectors.py
Handle option specification.

Author: GrimAndGreedy
License: MIT
"""

import curses
from typing import Tuple
from listpick.ui.input_field import input_field
from listpick.utils.utils import dir_picker

def default_option_input(stdscr: curses.window, refresh_screen_function=None, starting_value:str="", field_prefix:str="Opts", registers={}) -> Tuple[bool, str]:
    # notification(stdscr, message=f"opt required for {index}")
    usrtxt = f"{starting_value} " if starting_value else ""
    h, w = stdscr.getmaxyx()
    # field_end = w-38 if show_footer else w-3
    field_end = w-3
    field_end_f = lambda: stdscr.getmaxyx()[1]-3
    usrtxt, return_val = input_field(
        stdscr,
        usrtxt=usrtxt,
        field_prefix=field_prefix,
        x=lambda:2,
        y=lambda: stdscr.getmaxyx()[0]-1,
        max_length=field_end_f,
        registers=registers,
    )
    if return_val: return True, usrtxt
    else: return False, starting_value



def default_option_selector(stdscr: curses.window, refresh_screen_function=None, starting_value:str="", field_prefix:str=" Opts: ", registers={}) -> Tuple[bool, str]:
    """ 
    *** **** *** ** ** *
    NOT YET IMPLEMENTED!!
    *** **** *** ** ** *
    """
    # notification(stdscr, message=f"opt required for {index}")
    usrtxt = f"{starting_value} " if starting_value else ""
    h, w = stdscr.getmaxyx()
    # field_end = w-38 if show_footer else w-3
    field_end = w-3
    field_end_f = lambda: stdscr.getmaxyx()[1]-3
    usrtxt, return_val = input_field(
        stdscr,
        usrtxt=usrtxt,
        field_prefix=field_prefix,
        x=lambda:2,
        y=lambda: stdscr.getmaxyx()[0]-1,
        max_length=field_end_f,
        registers=registers,
    )
    if return_val: return True, usrtxt
    else: return False, starting_value


def output_file_option_selector(stdscr:curses.window, refresh_screen_function, registers={}) -> Tuple[bool, str]:
    s = dir_picker()

    stdscr.clear()
    stdscr.refresh()
    refresh_screen_function()
    usrtxt = f"{s}/"
    h, w = stdscr.getmaxyx()
    # field_end = w-38 if show_footer else w-3
    field_end_f = lambda: stdscr.getmaxyx()[1]-3
    usrtxt, return_val = input_field(
        stdscr,
        usrtxt=usrtxt,
        field_prefix=" Save as: ",
        x=lambda:2,
        y=lambda: stdscr.getmaxyx()[0]-1,
        max_length=field_end_f,
        registers=registers,
    )
    if return_val: return True, usrtxt
    else: return False, ""



#!/bin/python
# -*- coding: utf-8 -*-
"""
filtering.py
Apply filter(s) to a list of lists and return indexed matched rows.

Author: GrimAndGreedy
License: MIT
"""

import re
from typing import Tuple
from listpick.utils.search_and_filter_utils import apply_filter, tokenise
import os

def filter_items(items: list[list[str]], indexed_items: list[Tuple[int, list[str]]], query: str) -> list[Tuple[int, list[str]]]:
    """ 
    Filter items based on the query.

    Accepts:
        regular expressions
        --# to specify column to match
        --i to specify case-sensitivity (it is case insensitive by default)
        --v to specify inverse match

    E.g.,

        --1 query       matches query in the 1 column


    Returns indexed_items, which is a list of tuples; each tuple consists of the index and the data of the matching row in the original items list. 
    """


    invert_filter = False
    case_sensitive = False

    filters = tokenise(query)

    indexed_items = [(i, item) for i, item in enumerate(items) if apply_filter(item, filters)]
    return indexed_items



import sys, os
from listpick.listpick_app import Picker, close_curses, start_curses

l = [["The", "many"], ["words", "of"]]
l += [["the", "Athenians"], ["I", "do"]]
l += [["not", "understand."], ["They", "said"]]
l += [["a great", "deal"], ["in praise", "of themselves"]]
l += [["but", "nowhere"], ["denied", "that they"]]
l += [["are", "injuring"], ["our", "allies"]]

header=["Pericles", "Is Dead"]

highlights = [
    {
        "match": "praise",
        "field": 0,
        "color": 8,
    },
    {
        "match": "theni..",
        "field": 1,
        "color": 9,
    },
    {
        "match": ".*",
        "row":   5,
        "field": 1,
        "color": 11,
    },
]

stdscr = start_curses()
x = Picker(
        stdscr = stdscr,
        items = l,
        title="Test Picker",
        header=header,
        highlights=highlights,
    )
selected_indices, opts, picker_data = x.run()

close_curses(stdscr)

print(f"Selected: {selected_indices}")
print(f"Opts: {opts}")
